@@ -37,19 +37,6 @@ my ($auto_select, $no_install, $install_src, $clean, $noclean, $force, $parallel
 my ($ok, $nok);
 my $exit_code;
 
-
-=head1 NAME
-
-urpm::main_loop - The install/remove main loop for urpm based programs (urpmi, gurpmi, rpmdrake, drakx)
-
-=head1 SYNOPSIS
-
-=head1 DESCRIPTION
-
-=over
-
-=cut
-
 sub _download_callback {
     my ($urpm, $callbacks, $raw_msg, $msg) = @_;
     if (my $download_errors = delete $urpm->{download_errors}) {
@@ -98,12 +85,12 @@ sub _download_all {
     my ($urpm, $blists, $sources, $callbacks) = @_;
     if ($urpm->{options}{'download-all'}) {
         $urpm->{cachedir} = $urpm->{'urpmi-root'} . $urpm->{options}{'download-all'};
-        urpm::init_dir($urpm, $urpm->{cachedir});
+        urpm::init_cache_dir($urpm, $urpm->{cachedir});
     }
     my (undef, $available) = urpm::sys::df("$urpm->{cachedir}/rpms");
 
     if (!$urpm->{options}{ignoresize}) {
-        my ($download_size) = urpm::get_pkgs::get_distant_media_filesize($blists, $sources); 
+        my ($download_size) = urpm::get_pkgs::get_distant_media_filesize($urpm, $blists, $sources); 
         if ($download_size >= $available*1000) {
             my $p = N("There is not enough space on your filesystem to download all packages (%s needed, %s available).\nAre you sure you want to continue?", formatXiB($download_size), formatXiB($available*1000)); 
             $force || urpm::msg::ask_yes_or_no($p) or return 10;
@@ -163,16 +150,6 @@ sub _install_src {
     }
 }
 
-sub clean_trans_sources_from_src_packages {
-    my ($urpm, $transaction_sources_install, $transaction_sources) = @_;
-    foreach ($transaction_sources_install, $transaction_sources) {
-        foreach my $id (keys %$_) {
-            my $pkg = $urpm->{depslist}[$id] or next;
-            $pkg->arch eq 'src' and delete $_->{$id};
-        }
-    }
-}
-
 sub _continue_on_error {
     my ($urpm, $callbacks, $msgs, $error_sources, $formatted_errors) = @_;
     my $go_on;
@@ -217,11 +194,10 @@ sub _init_common_options {
       justdb => $options{justdb},
       replacepkgs => $options{replacepkgs},
       callback_close_helper => $callbacks->{close_helper},
-      callback_error => $callbacks->{error},
       callback_inst => $callbacks->{inst},
       callback_open_helper => $callbacks->{open_helper},
       callback_trans => $callbacks->{trans},
-      callback_uninst => $callbacks->{uninst},
+      callback_report_uninst => $callbacks->{callback_report_uninst},
       raw_message => 1,
   );
 }
@@ -310,137 +286,9 @@ sub _run_transaction {
     !$fatal;
 }
 
-=item run($urpm, $state, $something_was_to_be_done, $ask_unselect, $callbacks)
-
-Run the main urpm loop:
-
-=over
-
-=item * mount removable media if needed
-
-=item * split the work in smaller transactions
-
-=item * for each transaction:
-
-=over
-
-=item * prepare the transaction
-
-=item * download packages needed for this small transaction
-
-=item * verify packages
-
-=item * split package that should be installed instead of upgraded,
-
-=item * install source package only (whatever the user is root or not, but use rpm for that)
-
-=item * install/remove other packages
-
-=back
-
-=item * migrate the chrooted rpmdb if needed
-
-=item * display the final success/error message(s)
-
-=back
-
-Warning: locking is left to callers...
-
-Parameters:
-
-=over
-
-=item $urpm: the urpm object
-
-=item $state: the state object (see L<URPM>)
-
-=item $something_was_to_be_done
-
-=item $ask_unselect: an array ref of packages that could not be selected
-
-=item $callbacks: a hash ref of callbacks :
-
-=over
-
-=item trans_log($mode, $file, $percent, $total, $eta, $speed): called for displaying download progress
-
-=item interaction with user:
-
-=over
-
-=item ask_yes_or_no($_title, $msg)
-
-=item need_restart($need_restart_formatted) called when restarting urpmi is needed (priority upgrades)
-
-=item message($_title, $msg): display a message (with a title for GUIes)
-
-=back
-
-=item signature management:
-
-=over
-
-=item pre_check_sig(): signature checking startup (for rpmdrake)
-
-=item check_sig(): signature checking progress (for rpmdrake)
-
-=item bad_signature($msg, $msg2): called when a package is not/badly signed
-
-=back
-
-=item removable media management:
-
-=over
-
-=item pre_removable(): called before handling removable media (for rpmdrake)
-
-=item copy_removable($medium_name): called for asking inserting a CD/DVD
-
-=item post_extract($set, $transaction_sources, $transaction_sources_install) called after handling removable media (for rpmdrake)
-
-=back
-
-=item packages installation callbacks (passed to urpm::install::install(), see L<URPM> for parameters)
-
-=over
-
-=item open_helper(): called when opening a package, must return a fd
-
-=item close_helper(): called when package is closed
-
-=item inst() called for package opening/progress/end
-
-=item trans() called for transaction opening/progress/end
-
-=item uninst(): called for erasure progress
-
-=item error() called for cpio, script or unpacking errors
-
-=back
-
-=item finish callbacks (mainly GUI callbacks for rpmdrake/gurpmi/drakx)
-
-=over
-
-=item completed(): called when everything is completed (for cleanups)
-
-=item trans_error_summary($nok, $errors) called to warn than $nok transactions failed with $errors messages
-
-=item success_summary() called on success
-
-=item already_installed_or_not_installable($msg1, $msg2)
-
-=back
-
-=back
-
-=back
-
-=cut
-
-
+# locking is left to callers
 sub run {
-    my ($urpm, $state, $something_was_to_be_done, $ask_unselect, $callbacks) = @_;
+    my ($urpm, $state, $something_was_to_be_done, $ask_unselect, $_requested, $callbacks) = @_;
 
     #- global boolean options
     ($auto_select, $no_install, $install_src, $clean, $noclean, $force, $parallel, $test) =
@@ -490,14 +338,14 @@ sub run {
     my $migrate_back_rpmdb_db_version = 
       $urpm->{root} && urpm::select::should_we_migrate_back_rpmdb_db_version($urpm, $state);
 
-    #- now process each remove/install transaction
+    #- no process each remove/install transaction
     foreach my $set (@{$state->{transaction} || []}) {
 
         #- put a blank line to separate with previous transaction or user question.
         $urpm->{print}("\n") if $options{verbose} >= 0;
 
         my ($transaction_blists, $transaction_sources) = 
-          urpm::install::prepare_transaction($set, $blists, \%sources);
+          urpm::install::prepare_transaction($urpm, $set, $blists, \%sources);
 
         #- first, filter out what is really needed to download for this small transaction.
         my ($error_sources, $msgs) = _download_packages($urpm, $callbacks, $transaction_blists, $transaction_sources);
@@ -508,8 +356,8 @@ sub run {
 
         $callbacks->{post_download} and $callbacks->{post_download}->();
 
-        #- extract packages that should be installed instead of upgraded,
-        my %transaction_sources_install = %{$urpm->extract_packages_to_install($transaction_sources) || {}};
+        #- extract package that should be installed instead of upgraded,
+        my %transaction_sources_install = %{$urpm->extract_packages_to_install($transaction_sources, $state) || {}};
         $callbacks->{post_extract} and $callbacks->{post_extract}->($set, $transaction_sources, \%transaction_sources_install);
 
         #- verify packages
@@ -527,7 +375,12 @@ sub run {
         next if $no_install;
 
         #- clean to remove any src package now.
-        clean_trans_sources_from_src_packages($urpm, \%transaction_sources_install, $transaction_sources);
+        foreach (\%transaction_sources_install, $transaction_sources) {
+            foreach my $id (keys %$_) {
+                my $pkg = $urpm->{depslist}[$id] or next;
+                $pkg->arch eq 'src' and delete $_->{$id};
+            }
+        }
 
         #- install/remove other packages
         if (keys(%transaction_sources_install) || keys(%$transaction_sources) || $set->{remove}) {
@@ -616,15 +469,3 @@ sub handle_need_restart {
 }
 
 1;
-
-=back
-
-=head1 COPYRIGHT
-
-Copyright (C) 1999-2005 MandrakeSoft SA
-
-Copyright (C) 2005-2010 Mandriva SA
-
-Copyright (C) 2011-2013 Mageia
-
-=cut
